<?php

namespace PayrexxPaymentGateway\Service;

use Payrexx\Models\Response\Transaction;

class OrderService
{
    /**
     * @param Transaction $payrexxTransaction
     */
    public function handleTransactionStatus($order, array $subscriptions, $newTransactionStatus, $preAuthId, string $paymentMethod = '') {
        switch ($newTransactionStatus) {
            case Transaction::WAITING:
                $newTransactionsStatus = 'on-hold';
                $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);

                if ($order->get_status() === $newTransactionsStatus) {
                    break;
                }

                $order->update_status($newTransactionsStatus, __('Awaiting payment', 'wc-payrexx-gateway'));
                break;
            case Transaction::CONFIRMED:
                $this->setOrderPaid($order, $paymentMethod);
                break;
            case Transaction::AUTHORIZED:
                foreach ($subscriptions as $subscription) {
                    $subscription->update_meta_data('payrexx_auth_transaction_id', $preAuthId);
                    $subscription->update_meta_data('payrexx_payment_method', $paymentMethod);
                    $subscription->save();
                }

                // An order with amount 0 is considered as paid if the authorization is successful
                if (floatval($order->get_total('edit')) === 0.0) {
                    $this->setOrderPaid($order, $paymentMethod);
                }
                break;
            case Transaction::REFUNDED:
                $newTransactionsStatus = 'refunded';
                $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);
                if ($order->get_status() === $newTransactionsStatus) {
                    break;
                }
                $order->update_status($newTransactionsStatus, __('Payment was fully refunded', 'wc-payrexx-gateway'));
                break;
//                    case Transaction::PARTIALLY_REFUNDED:
//                        if ($order->get_status() === 'refunded') {
//                            break;
//                        }
//                        $order->update_status('refunded', __('Payment was partially refunded', 'wc-payrexx-gateway'));
//                        break;
            case Transaction::CANCELLED:
            case Transaction::EXPIRED:
            case Transaction::DECLINED:
                $newTransactionsStatus = 'cancelled';
                $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);

                if ($order->has_status(['cancelled', 'processing', 'completed', $newTransactionsStatus])) {
                    break;
                }

                $order->update_status($newTransactionsStatus, __('Payment was cancelled by the customer', 'wc-payrexx-gateway'));
                break;
            case Transaction::ERROR:
                $newTransactionsStatus = 'failed';
                $newTransactionsStatus = $this->getCustomTransactionStatus($newTransactionsStatus);
                if ($order->has_status(['failed', 'processing', 'completed', $newTransactionsStatus])) {
                    break;
                }
                $order->update_status($newTransactionsStatus, __('An error occured while processing this payment', 'wc-payrexx-gateway'));
                break;
        }
    }

    private function setOrderPaid($order, $paymentBrand) {
        if ($order->has_status(['processing', 'completed'])) {
            return;
        }

        if ($paymentBrand) {
            $order->set_payment_method(ucfirst($paymentBrand));
        }

        $order->payment_complete();
        // Remove cart
        WC()->cart->empty_cart();
    }

    private function getCustomTransactionStatus($status) {
        return apply_filters('woo_payrexx_custom_transaction_status_' . $status, $status);
    }
}