<?php

namespace PayrexxPaymentGateway\Service;

use Payrexx\Communicator;
use Payrexx\Models\Response\Gateway;
use Payrexx\Models\Response\Transaction;

class PayrexxApiService
{
    private $instance;
    private $apiKey;
    private $platform;
    private $prefix;
    private $lookAndFeelId;

    /**
     * Constructor
     *
     * @param EntityRepository $customerRepository
     * @param LoggerInterface $logger
     */
    public function __construct($instance, $apiKey, $platform, $prefix, $lookAndFeelId)
    {
        $this->instance = $instance;
        $this->apiKey = $apiKey;
        $this->platform = $platform;
        $this->prefix = $prefix;
        $this->lookAndFeelId = $lookAndFeelId;
    }

    public function createPayrexxGateway($order, $totalAmount, $basket, $purpose, $reference, $successRedirectUrl, $cancelRedirectUrl, $preAuthorization, $chargeOnAuth, $currency = 'USD', $customButtonText = null) {
        $payrexx = $this->getInterface();
        $gateway = new \Payrexx\Models\Request\Gateway();

        $gateway->setValidity(15);
        $gateway->setPsp([]);
        $gateway->setSkipResultPage(true);

        if ($customButtonText) {
            $gateway->setButtonText($customButtonText);
        }

        $gateway->setAmount($totalAmount * 100);
        $gateway->setCurrency($currency);
        $gateway->setPreAuthorization($preAuthorization);
        $gateway->setChargeOnAuthorization($chargeOnAuth);
        $gateway->setBasket($basket);
        $gateway->setPurpose($purpose);
        $gateway->setReferenceId($reference);

        $gateway->setLookAndFeelProfile($this->lookAndFeelId ?: null);

        $gateway->setSuccessRedirectUrl($successRedirectUrl);
        $gateway->setCancelRedirectUrl($cancelRedirectUrl);
        $gateway->setFailedRedirectUrl($cancelRedirectUrl);

        $billingAddress = $order->get_billing_address_1() . ' ' . $order->get_billing_address_2();
        $gateway->addField($type = 'title', $value = '');
        $gateway->addField($type = 'forename', $value = $order->get_billing_first_name());
        $gateway->addField($type = 'surname', $value = $order->get_billing_last_name());
        $gateway->addField($type = 'company', $value = $order->get_billing_company());
        $gateway->addField($type = 'street', $value = $billingAddress);
        $gateway->addField($type = 'postcode', $value = $order->get_billing_postcode());
        $gateway->addField($type = 'place', $value = $order->get_billing_city());
        $gateway->addField($type = 'country', $value = $order->get_billing_country());
        $gateway->addField($type = 'phone', $value = $order->get_billing_phone());
        $gateway->addField($type = 'email', $value = $order->get_billing_email());
        $gateway->addField($type = 'custom_field_1', $value = $order->get_id(), $name = 'WooCommerce Order ID');

        try {
            $response = $payrexx->create($gateway);

            return $response;
        } catch (\Payrexx\PayrexxException $e) {
            print $e->getMessage();
        }
    }

    /**
     * @return \Payrexx\Payrexx
     */
    public function getInterface(): \Payrexx\Payrexx
    {
        $platform = !empty($this->platform) ? $this->platform : \Payrexx\Communicator::API_URL_BASE_DOMAIN;
        return new \Payrexx\Payrexx($this->instance, $this->apiKey, '', $platform);
    }

    public function deleteGatewayById($gatewayId):bool {
        $payrexx = $this->getInterface();

        $gateway = new \Payrexx\Models\Request\Gateway();
        $gateway->setId($gatewayId);

        try {
            $payrexx->delete($gateway);
        } catch (\Payrexx\PayrexxException $e) {
            return false;
        }
        return true;
    }

    public function getPayrexxTransaction(int $payrexxTransactionId): ?\Payrexx\Models\Response\Transaction
    {
        $payrexx = $this->getInterface();

        $payrexxTransaction = new \Payrexx\Models\Request\Transaction();
        $payrexxTransaction->setId($payrexxTransactionId);

        try {
            $response = $payrexx->getOne($payrexxTransaction);
            return $response;
        } catch (\Payrexx\PayrexxException $e) {
            return null;
        }
    }

    public function chargeTransaction($transactionId, $amount) {
        $payrexx = $this->getInterface();
        $transaction = new \Payrexx\Models\Request\Transaction();
        $transaction->setId($transactionId);
        $transaction->setAmount(floatval($amount) * 100);
        try {
            $payrexx->charge($transaction);
            return true;
        } catch (\Payrexx\PayrexxException $e) {
        }
        return false;
    }

    /**
     * @param $gatewayId
     * @return \Payrexx\Models\Request\Gateway
     */
    public function getPayrexxGateway($gatewayId) {
        $payrexx = $this->getInterface();
        $gateway = new \Payrexx\Models\Request\Gateway();
        $gateway->setId($gatewayId);
        try {
            $payrexxGateway = $payrexx->getOne($gateway);
            return $payrexxGateway;
        } catch (\Payrexx\PayrexxException $e) {
            throw new \Exception('No gateway found by ID: '. $gatewayId);
        }
    }
}